# frozen_string_literal: true

require "jsonapi/types"
require "http"

require "jsonapi/simple_client/errors"
require "jsonapi/simple_client/query_params"
require "jsonapi/simple_client/request"
require "jsonapi/simple_client/version"

module JSONAPI
  module SimpleClient
    # @see http://jsonapi.org/format/#crud-creating
    # @param url [String]
    # @param headers [Hash]
    # @param payload [Hash]
    # @return [JSONAPI::Types::Success, JSONAPI::Types::Document, JSONAPI::Types::Failure]
    def self.create(url, headers = {}, payload = {})
      response = Request.new(url, headers).post(payload)

      case response.status
      when 201
        JSONAPI::Types::Success.parse(response.to_s)
      when 202, 204
        JSONAPI::Types::Document.parse(response.to_s)
      when 400, 422
        JSONAPI::Types::Failure.parse(response.to_s)
      end
    end

    # @see http://jsonapi.org/format/#crud-updating
    # @param url [String]
    # @param headers [Hash]
    # @param payload [Hash]
    # @return [JSONAPI::Types::Success, JSONAPI::Types::Document, JSONAPI::Types::Failure]
    def self.update(url, headers = {}, payload = {})
      response = Request.new(url, headers).patch(payload)

      case response.status
      when 200
        JSONAPI::Types::Success.parse(response.to_s)
      when 202, 204
        JSONAPI::Types::Document.parse(response.to_s)
      when 400, 422
        JSONAPI::Types::Failure.parse(response.to_s)
      end
    end

    # @see http://jsonapi.org/format/#crud-deleting
    # @param url [String]
    # @param headers [Hash]
    # @return [JSONAPI::Types::Info, JSONAPI::Types::Document, JSONAPI::Types::Failure]
    def self.delete(url, headers = {})
      response = Request.new(url, headers).delete

      case response.status
      when 200
        JSONAPI::Types::Info.parse(response.to_s)
      when 202, 204
        JSONAPI::Types::Document.parse(response.to_s)
      when 400, 422
        JSONAPI::Types::Failure.parse(response.to_s)
      end
    end

    # @see http://jsonapi.org/format/#fetching
    # @param url [String]
    # @param headers [Hash]
    # @param params [Hash]
    # @return [JSONAPI::Types::Success, JSONAPI::Types::Failure]
    def self.fetch(url, headers = {}, params = {})
      response = Request.new(url, headers).get(params)

      case response.status
      when 200
        JSONAPI::Types::Success.parse(response.to_s)
      when 400
        JSONAPI::Types::Failure.parse(response.to_s)
      end
    end
  end
end
