# frozen_string_literal: true

module JSONAPI
  module SimpleClient
    class Request
      CONTENT_TYPE = "application/vnd.api+json"

      attr_reader :url, :headers

      # @param url [String]
      # @param headers [Hash]
      # @return [HTTP::Response]
      def initialize(url, headers = {})
        @headers = headers
        @url = url
      end

      # @param payload [Hash]
      # @raise [JSONAPI::SimpleClient::Error] if the response is not parseable.
      # @return [HTTP::Response] if the request is successfull or a parseable error.
      def post(payload = {})
        execute { http_client.post(url, json: payload) }
      end

      # @param payload [Hash]
      # @raise [JSONAPI::SimpleClient::Error] if the response is not parseable.
      # @return [HTTP::Response] if the request is successfull or a parseable error.
      def patch(payload = {})
        execute { http_client.patch(url, json: payload) }
      end

      # @raise [JSONAPI::SimpleClient::Error] if the response is not parseable.
      # @return [HTTP::Response] if the request is successfull or a parseable error.
      def delete
        execute { http_client.delete(url) }
      end

      # @param params [Hash]
      # @raise [JSONAPI::SimpleClient::Error] if the response is not parseable.
      # @return [HTTP::Response] if the request is successfull or a parseable error.
      def get(params = {})
        execute { http_client.get(url, params: JSONAPI::SimpleClient::QueryParams.parse(params)) }
      end

      private

      # @return [HTTP]
      def http_client
        HTTP
          .accept(CONTENT_TYPE)
          .headers(headers.merge(content_type: CONTENT_TYPE))
      end

      # @raise [JSONAPI::SimpleClient::Error] if the response is not parseable.
      # @return [HTTP::Response] if the request is successfull or a parseable error.
      def execute
        response = yield
        response.status.success? ? response : handle_error(response)
      end

      # @param response [HTTP::Response]
      # @raise [JSONAPI::SimpleClient::Error] if the response is not a parseable error.
      # @return [HTTP::Response]
      def handle_error(response)
        case response.status
        when 400, 422 then response
        else
          raise error_class_for(response.status)
        end
      end

      # @param status [Integer]
      # @return [JSONAPI::SimpleClient::Error]
      def error_class_for(status)
        {
          403 => JSONAPI::SimpleClient::Forbidden,
          404 => JSONAPI::SimpleClient::NotFound,
          409 => JSONAPI::SimpleClient::Conflict,
          500 => JSONAPI::SimpleClient::ServerError
        }.fetch(status, JSONAPI::SimpleClient::Error)
      end
    end
  end
end
